# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/ransim/trafficsim/trafficsim.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator

import betterproto
import grpclib


class Type(betterproto.Enum):
    """Change event type"""

    # NONE indicates this response does not represent a modification of the
    # Change
    NONE = 0
    # ADDED is an event which occurs when a Change is added to the topology
    ADDED = 1
    # UPDATED is an event which occurs when a Change is updated
    UPDATED = 2
    # REMOVED is an event which occurs when a Change is removed from the
    # configuration
    REMOVED = 3


class UpdateType(betterproto.Enum):
    """
    In the case of an update this helps qualify the update type - add items as
    necessary
    """

    NOUPDATETYPE = 0
    # POSITION An update in position of UE only, without a change in the Tower
    # affinity
    POSITION = 1
    # TOWER An update in non-serving tower affinity (and also position) of UE
    TOWER = 2
    # HANDOVER An update of the serving tower
    HANDOVER = 3


@dataclass(eq=False, repr=False)
class MapLayoutRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListRoutesRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListRoutesResponse(betterproto.Message):
    # route is the route change on which the event occurred
    route: "_types__.Route" = betterproto.message_field(1)
    # type is a qualification of the type of change being made
    type: "Type" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListUesRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListUesResponse(betterproto.Message):
    # Ue is the UserEquipment change on which the event occurred
    ue: "_types__.Ue" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchUesRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchUesResponse(betterproto.Message):
    # Ue is the UserEquipment change on which the event occurred
    ue: "_types__.Ue" = betterproto.message_field(1)
    # type is a qualification of the type of change being made
    type: "Type" = betterproto.enum_field(2)
    # update_type is a qualification of the type of UE change
    update_type: "UpdateType" = betterproto.enum_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SetNumberUEsRequest(betterproto.Message):
    number: int = betterproto.uint32_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SetNumberUEsResponse(betterproto.Message):
    number: int = betterproto.uint32_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ResetMetricsMsg(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


class TrafficStub(betterproto.ServiceStub):
    """Traffic - provides a stream of traffic data to GUI"""

    async def get_map_layout(self) -> "_types__.MapLayout":

        request = MapLayoutRequest()

        return await self._unary_unary(
            "/onos.ransim.trafficsim.Traffic/GetMapLayout", request, _types__.MapLayout
        )

    async def list_routes(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["ListRoutesResponse"]:

        request = ListRoutesRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.trafficsim.Traffic/ListRoutes",
            request,
            ListRoutesResponse,
        ):
            yield response

    async def list_ues(self) -> AsyncIterator["ListUesResponse"]:

        request = ListUesRequest()

        async for response in self._unary_stream(
            "/onos.ransim.trafficsim.Traffic/ListUes",
            request,
            ListUesResponse,
        ):
            yield response

    async def watch_ues(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchUesResponse"]:

        request = WatchUesRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.trafficsim.Traffic/WatchUes",
            request,
            WatchUesResponse,
        ):
            yield response

    async def set_number_u_es(self, *, number: int = 0) -> "SetNumberUEsResponse":

        request = SetNumberUEsRequest()
        request.number = number

        return await self._unary_unary(
            "/onos.ransim.trafficsim.Traffic/SetNumberUEs",
            request,
            SetNumberUEsResponse,
        )

    async def reset_metrics(self) -> "ResetMetricsMsg":

        request = ResetMetricsMsg()

        return await self._unary_unary(
            "/onos.ransim.trafficsim.Traffic/ResetMetrics", request, ResetMetricsMsg
        )


from .. import types as _types__



#!/usr/bin/env python3
# Copyright 2004-present Facebook. All Rights Reserved.

from typing import Optional


class AioMsaError(Exception):
    """Base class for all *aiomsa* errors."""

    pass


class DuplicateRouteError(AioMsaError):
    """Raised if more than one route has the same method and path.

    Args:
        method: The ``HTTP`` method used by the route.
        path: The ``URL`` path used by the route.
        msg: The exception message.
    """

    def __init__(self, method: str, path: str, msg: Optional[str] = None):
        if msg is None:
            msg = f"A route with method, {method}, and path, {path}, already exists"
        super().__init__(msg)
        self.method = method
        self.path = path


class ClientError(AioMsaError):
    """Base class for all client errors."""

    pass


class ClientRestartError(ClientError):
    """Raised if a client is started while already running.

    Args:
        msg: The exception message.
    """

    def __init__(self, msg: Optional[str] = None):
        if msg is None:
            msg = "Cannot start client more than once"
        super().__init__(msg)


class ClientStoppedError(ClientError):
    """Raised if the client is consumed when not running.

    Args:
        msg: The exception message.
    """

    def __init__(self, msg: Optional[str] = None):
        if msg is None:
            msg = "Client is not currently running"
        super().__init__(msg)


class ClientRuntimeError(ClientError):
    """Raised if client behavior fails for any other reason after initialization.

    Args:
        msg: The exception message.
    """

    def __init__(self, msg: Optional[str] = None):
        if msg is None:
            msg = "An unknown issue with the client occurred at runtime"
        super().__init__(msg)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import json

import aiohttp.web
import yaml
from aiohttp_swagger import generate_doc_from_each_end_point

from aiomsa.server import routes


# -- Module Remapping --------------------------------------------------------

aiohttp.web.Application.__module__ = "aiohttp.web"


# -- Project information -----------------------------------------------------

project = "aiomsa"
copyright = "2021, Facebook Connectivity"
author = "Facebook Connectivity"

# The full version, including alpha/beta/rc tags
release = "0.1.0a1"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinx.ext.napoleon",
    "sphinx_autodoc_typehints",
    "sphinxcontrib.openapi",
]

# Add any paths that contain templates here, relative to this directory.
# templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# A list of (type, target) tuples (by default empty) that should be ignored
# when generating warnings in “nitpicky mode”
nitpick_ignore = [("http:obj", "string")]  # sphinxcontrib.openapi


# -- intersphinx configuration -----------------------------------------------

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "aiohttp": ("https://docs.aiohttp.org/en/stable/", None),
}


# -- autodoc configuration ---------------------------------------------------

autodoc_member_order = "bysource"


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "furo"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ["_static"]


def setup(app):
    web_app = aiohttp.web.Application()
    web_app.add_routes(routes)
    with open("openapi.yml", "w+") as f:
        docs = json.loads(generate_doc_from_each_end_point(web_app, ui_version=3))
        yaml.dump(docs, f)

    return app



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2t/admin/admin.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterable, AsyncIterator, Iterable, List, Optional, Union

import betterproto
import grpclib


class E2NodeConnectionType(betterproto.Enum):
    """E2NodeConnectionType specifies the type of an E2 connection"""

    UNKNOWN = 0
    G_NB = 1
    E_NB = 2
    ENG_MB = 3
    NGE_NB = 4


@dataclass(eq=False, repr=False)
class UploadRegisterServiceModelRequest(betterproto.Message):
    """
    UploadRegisterServiceModelRequest is for streaming a model plugin file to
    the server. There is a built in limit in gRPC of 4MB - plugin is usually
    around 20MB so break in to chunks of approx 1-2MB.
    """

    # so_file is the name being streamed.
    so_file: str = betterproto.string_field(1)
    # content is the bytes content.
    content: bytes = betterproto.bytes_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UploadRegisterServiceModelResponse(betterproto.Message):
    """
    UploadRegisterServiceModelResponse carries status of model plugin
    registration.
    """

    # name is name of the model plugin.
    name: str = betterproto.string_field(1)
    # version is the semantic version of the model plugin.
    version: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListRegisteredServiceModelsResponse(betterproto.Message):
    """
    ListRegisteredServiceModelsResponse is general information about a service
    model plugin.
    """

    # name is the name given to the service model plugin - no spaces and title
    # case.
    name: str = betterproto.string_field(1)
    # version is the semantic version of the Plugin e.g. 1.0.0.
    version: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListRegisteredServiceModelsRequest(betterproto.Message):
    """
    ListRegisteredServiceModelsRequest carries data for querying registered
    service model plugins.
    """

    # An optional filter on the name of the model plugins to list.
    model_name: str = betterproto.string_field(1)
    # An optional filter on the version of the model plugins to list
    model_version: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListE2NodeConnectionsRequest(betterproto.Message):
    """
    ListE2NodeConnectionsRequest carries request for a list of E2 node SCTP
    connections.
    """

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListE2NodeConnectionsResponse(betterproto.Message):
    """
    ListE2NodeConnectionsResponse carries information about the SCTP connection
    to the remote E2 node.
    """

    remote_ip: List[str] = betterproto.string_field(1)
    remote_port: int = betterproto.uint32_field(2)
    id: str = betterproto.string_field(3)
    plmn_id: str = betterproto.string_field(4)
    connection_type: "E2NodeConnectionType" = betterproto.enum_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DropE2NodeConnectionsRequest(betterproto.Message):
    """DropE2NodeConnectionsRequest carries drop connection request"""

    connections: List["ListE2NodeConnectionsResponse"] = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DropE2NodeConnectionsResponse(betterproto.Message):
    """DropE2NodeConnectionsResponse carries drop connection response"""

    success: List[bool] = betterproto.bool_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


class E2TAdminServiceStub(betterproto.ServiceStub):
    """
    E2TAdminService provides means for enhanced interactions with the ONOS RIC
    E2 Termination service.
    """

    async def upload_register_service_model(
        self,
        request_iterator: Union[
            AsyncIterable["UploadRegisterServiceModelRequest"],
            Iterable["UploadRegisterServiceModelRequest"],
        ],
    ) -> "UploadRegisterServiceModelResponse":
        """
        UploadRegisterServiceModel uploads and adds the model plugin to the
        list of supported models. The file is serialized in to Chunks of less
        than 4MB so as not to break the gRPC byte array limit
        """

        return await self._stream_unary(
            "/onos.e2t.admin.E2TAdminService/UploadRegisterServiceModel",
            request_iterator,
            UploadRegisterServiceModelRequest,
            UploadRegisterServiceModelResponse,
        )

    async def list_registered_service_models(
        self, *, model_name: str = "", model_version: str = ""
    ) -> AsyncIterator["ListRegisteredServiceModelsResponse"]:
        """
        ListRegisteredServiceModels returns a stream of registered service
        models.
        """

        request = ListRegisteredServiceModelsRequest()
        request.model_name = model_name
        request.model_version = model_version

        async for response in self._unary_stream(
            "/onos.e2t.admin.E2TAdminService/ListRegisteredServiceModels",
            request,
            ListRegisteredServiceModelsResponse,
        ):
            yield response

    async def list_e2_node_connections(
        self,
    ) -> AsyncIterator["ListE2NodeConnectionsResponse"]:
        """
        ListE2NodeConnections returns a stream of existing SCTP connections.
        """

        request = ListE2NodeConnectionsRequest()

        async for response in self._unary_stream(
            "/onos.e2t.admin.E2TAdminService/ListE2NodeConnections",
            request,
            ListE2NodeConnectionsResponse,
        ):
            yield response

    async def drop_e2_node_connections(
        self, *, connections: Optional[List["ListE2NodeConnectionsResponse"]] = None
    ) -> "DropE2NodeConnectionsResponse":
        """
        DropE2NodeConnections drops the specified E2 node SCTP connections
        """

        connections = connections or []

        request = DropE2NodeConnectionsRequest()
        if connections is not None:
            request.connections = connections

        return await self._unary_unary(
            "/onos.e2t.admin.E2TAdminService/DropE2NodeConnections",
            request,
            DropE2NodeConnectionsResponse,
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/ransim/model/model.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, List, Optional

import betterproto
import grpclib


class EventType(betterproto.Enum):
    """Change event type"""

    # NONE indicates this response represents a pre-existing entity
    NONE = 0
    # CREATED indicates a new entity was created
    CREATED = 1
    # UPDATED indicates an existing entity was updated
    UPDATED = 2
    # DELETED indicates an entity was deleted
    DELETED = 3


@dataclass(eq=False, repr=False)
class PlmnIdRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class PlmnIdResponse(betterproto.Message):
    plmnid: int = betterproto.uint32_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CreateNodeRequest(betterproto.Message):
    """CreateNodeRequest create a node request"""

    node: "_types__.Node" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CreateNodeResponse(betterproto.Message):
    """CreateNodeResponse create a node response"""

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetNodeRequest(betterproto.Message):
    """GetNodeRequest get a node request"""

    enbid: int = betterproto.uint32_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetNodeResponse(betterproto.Message):
    """GetNodeResponse get a node response"""

    node: "_types__.Node" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateNodeRequest(betterproto.Message):
    """UpdateNodeRequest update a node request"""

    node: "_types__.Node" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateNodeResponse(betterproto.Message):
    """UpdateNodeResponse update a node response"""

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteNodeRequest(betterproto.Message):
    """DeleteNodeRequest delete a node request"""

    enbid: int = betterproto.uint32_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteNodeResponse(betterproto.Message):
    """DeleteNodeResponse delete a node response"""

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListNodesRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListNodesResponse(betterproto.Message):
    node: "_types__.Node" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchNodesRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchNodesResponse(betterproto.Message):
    node: "_types__.Node" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class AgentControlRequest(betterproto.Message):
    enbid: int = betterproto.uint32_field(1)
    command: str = betterproto.string_field(2)
    args: List[str] = betterproto.string_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class AgentControlResponse(betterproto.Message):
    node: "_types__.Node" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CreateCellRequest(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CreateCellResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetCellRequest(betterproto.Message):
    ecgi: int = betterproto.uint32_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetCellResponse(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateCellRequest(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateCellResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteCellRequest(betterproto.Message):
    enbid: int = betterproto.uint32_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteCellResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchCellsRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchCellsResponse(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListCellsRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListCellsResponse(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


class NodeModelStub(betterproto.ServiceStub):
    """
    Model provides means to create, delete and read RAN simulation model.
    """

    async def get_plmn_id(self) -> "PlmnIdResponse":

        request = PlmnIdRequest()

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/GetPlmnID", request, PlmnIdResponse
        )

    async def create_node(
        self, *, node: "_types__.Node" = None
    ) -> "CreateNodeResponse":

        request = CreateNodeRequest()
        if node is not None:
            request.node = node

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/CreateNode", request, CreateNodeResponse
        )

    async def get_node(self, *, enbid: int = 0) -> "GetNodeResponse":

        request = GetNodeRequest()
        request.enbid = enbid

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/GetNode", request, GetNodeResponse
        )

    async def update_node(
        self, *, node: "_types__.Node" = None
    ) -> "UpdateNodeResponse":

        request = UpdateNodeRequest()
        if node is not None:
            request.node = node

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/UpdateNode", request, UpdateNodeResponse
        )

    async def delete_node(self, *, enbid: int = 0) -> "DeleteNodeResponse":

        request = DeleteNodeRequest()
        request.enbid = enbid

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/DeleteNode", request, DeleteNodeResponse
        )

    async def watch_nodes(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchNodesResponse"]:

        request = WatchNodesRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.model.NodeModel/WatchNodes",
            request,
            WatchNodesResponse,
        ):
            yield response

    async def list_nodes(self) -> AsyncIterator["ListNodesResponse"]:

        request = ListNodesRequest()

        async for response in self._unary_stream(
            "/onos.ransim.model.NodeModel/ListNodes",
            request,
            ListNodesResponse,
        ):
            yield response

    async def agent_control(
        self, *, enbid: int = 0, command: str = "", args: Optional[List[str]] = None
    ) -> "AgentControlResponse":
        args = args or []

        request = AgentControlRequest()
        request.enbid = enbid
        request.command = command
        request.args = args

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/AgentControl", request, AgentControlResponse
        )


class CellModelStub(betterproto.ServiceStub):
    """
    Model provides means to create, delete and read RAN simulation model.
    """

    async def create_cell(
        self, *, cell: "_types__.Cell" = None
    ) -> "CreateCellResponse":

        request = CreateCellRequest()
        if cell is not None:
            request.cell = cell

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/CreateCell", request, CreateCellResponse
        )

    async def delete_cell(self, *, enbid: int = 0) -> "DeleteCellResponse":

        request = DeleteCellRequest()
        request.enbid = enbid

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/DeleteCell", request, DeleteCellResponse
        )

    async def update_cell(
        self, *, cell: "_types__.Cell" = None
    ) -> "UpdateCellResponse":

        request = UpdateCellRequest()
        if cell is not None:
            request.cell = cell

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/UpdateCell", request, UpdateCellResponse
        )

    async def get_cell(self, *, ecgi: int = 0) -> "GetCellResponse":

        request = GetCellRequest()
        request.ecgi = ecgi

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/GetCell", request, GetCellResponse
        )

    async def watch_cells(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchCellsResponse"]:

        request = WatchCellsRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.model.CellModel/WatchCells",
            request,
            WatchCellsResponse,
        ):
            yield response

    async def list_cells(self) -> AsyncIterator["ListCellsResponse"]:

        request = ListCellsRequest()

        async for response in self._unary_stream(
            "/onos.ransim.model.CellModel/ListCells",
            request,
            ListCellsResponse,
        ):
            yield response


from .. import types as _types__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/change/network/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class NetworkChange(betterproto.Message):
    """
    NetworkChange specifies the configuration for a network change A network
    change is a configuration change that spans multiple devices. The change
    contains a list of per-device changes to be applied to the network.
    """

    # 'id' is the unique identifier of the change This field should be set prior
    # to persisting the object.
    id: str = betterproto.string_field(1)
    # 'index' is a monotonically increasing, globally unique index of the change
    # The index is provided by the store, is static and unique for each unique
    # change identifier, and should not be modified by client code.
    index: int = betterproto.uint64_field(2)
    # 'revision' is the change revision number The revision number is provided by
    # the store and should not be modified by client code. Each unique state of
    # the change will be assigned a unique revision number which can be used for
    # optimistic concurrency control when updating or deleting the change state.
    revision: int = betterproto.uint64_field(3)
    # 'status' is the current lifecycle status of the change
    status: "__change__.Status" = betterproto.message_field(4)
    # 'created' is the time at which the change was created
    created: datetime = betterproto.message_field(5)
    # 'updated' is the time at which the change was last updated
    updated: datetime = betterproto.message_field(6)
    # 'changes' is a set of changes to apply to devices The list of changes
    # should contain only a single change per device/version pair.
    changes: List["_device__.Change"] = betterproto.message_field(7)
    # 'refs' is a set of references to stored device changes
    refs: List["DeviceChangeRef"] = betterproto.message_field(8)
    # 'deleted' is a flag indicating whether this change is being deleted by a
    # snapshot
    deleted: bool = betterproto.bool_field(9)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeviceChangeRef(betterproto.Message):
    """DeviceChangeRef is a reference to a device change"""

    # 'device_change_id' is the unique identifier of the device change
    device_change_id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


from .. import device as _device__
from ... import change as __change__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/ransim/metrics/metrics.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, List, Optional

import betterproto
import grpclib


class EventType(betterproto.Enum):
    """Change event type"""

    # NONE indicates unknown event type
    NONE = 0
    # UPDATED indicates a metric value was set (updated)
    UPDATED = 1
    # DELETED indicates a metric was deleted
    DELETED = 2


@dataclass(eq=False, repr=False)
class Metric(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    key: str = betterproto.string_field(2)
    value: str = betterproto.string_field(3)
    type: str = betterproto.string_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListResponse(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    metrics: List["Metric"] = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SetRequest(betterproto.Message):
    metric: "Metric" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SetResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    metric: "Metric" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteAllRequest(betterproto.Message):
    entityid: int = betterproto.uint64_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteAllResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchResponse(betterproto.Message):
    metric: "Metric" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


class MetricsServiceStub(betterproto.ServiceStub):
    """
    Model provides means to create, delete and read RAN simulation model.
    """

    async def list(self, *, entityid: int = 0) -> "ListResponse":
        """
        List returns an array of all metrics for the specified entity (Node,
        Cell or UE)
        """

        request = ListRequest()
        request.entityid = entityid

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/List", request, ListResponse
        )

    async def set(self, *, metric: "Metric" = None) -> "SetResponse":
        """Set sets value of the named metric for the specified entity"""

        request = SetRequest()
        if metric is not None:
            request.metric = metric

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/Set", request, SetResponse
        )

    async def get(self, *, entityid: int = 0, name: str = "") -> "GetResponse":
        """Get retrieves the named metric for the specified entity"""

        request = GetRequest()
        request.entityid = entityid
        request.name = name

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/Get", request, GetResponse
        )

    async def delete(self, *, entityid: int = 0, name: str = "") -> "DeleteResponse":
        """Delete deletes the the named metric for the specified entity"""

        request = DeleteRequest()
        request.entityid = entityid
        request.name = name

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/Delete", request, DeleteResponse
        )

    async def delete_all(self, *, entityid: int = 0) -> "DeleteAllResponse":
        """DeleteAll deletes all metrics for the specified entity"""

        request = DeleteAllRequest()
        request.entityid = entityid

        return await self._unary_unary(
            "/onos.ransim.metrics.MetricsService/DeleteAll", request, DeleteAllResponse
        )

    async def watch(self) -> AsyncIterator["WatchResponse"]:
        """Watch returns a stream of ongoing changes to the metrics"""

        request = WatchRequest()

        async for response in self._unary_stream(
            "/onos.ransim.metrics.MetricsService/Watch",
            request,
            WatchResponse,
        ):
            yield response



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/topo/topo.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict, List, Optional

import betterproto
import grpclib


class EventType(betterproto.Enum):
    """EventType is a topo operation event type"""

    NONE = 0
    ADDED = 1
    UPDATED = 2
    REMOVED = 3


class Protocol(betterproto.Enum):
    """Protocol to interact with a device"""

    # UNKNOWN_PROTOCOL constant needed to go around proto3 nullifying the 0
    # values
    UNKNOWN_PROTOCOL = 0
    # GNMI protocol reference
    GNMI = 1
    # P4RUNTIME protocol reference
    P4RUNTIME = 2
    # GNOI protocol reference
    GNOI = 3
    # E2 Control Plane Protocol
    E2AP = 4


class ConnectivityState(betterproto.Enum):
    """
    ConnectivityState represents the L3 reachability of a device from the
    service container (e.g. enos-config), independently of gRPC or the service
    itself (e.g. gNMI)
    """

    # UNKNOWN_CONNECTIVITY_STATE constant needed to go around proto3 nullifying
    # the 0 values
    UNKNOWN_CONNECTIVITY_STATE = 0
    # REACHABLE indicates the the service can reach the device at L3
    REACHABLE = 1
    # UNREACHABLE indicates the the service can't reach the device at L3
    UNREACHABLE = 2


class ChannelState(betterproto.Enum):
    """
    ConnectivityState represents the state of a gRPC channel to the device from
    the service container
    """

    # UNKNOWN_CHANNEL_STATE constant needed to go around proto3 nullifying the 0
    # values
    UNKNOWN_CHANNEL_STATE = 0
    # CONNECTED indicates the corresponding grpc channel is connected on this
    # device
    CONNECTED = 1
    # DISCONNECTED indicates the corresponding grpc channel is not connected on
    # this device
    DISCONNECTED = 2


class ServiceState(betterproto.Enum):
    """
    ServiceState represents the state of the gRPC service (e.g. gNMI) to the
    device from the service container
    """

    # UNKNOWN_SERVICE_STATE constant needed to go around proto3 nullifying the 0
    # values
    UNKNOWN_SERVICE_STATE = 0
    # AVAILABLE indicates the corresponding grpc service is available
    AVAILABLE = 1
    # UNAVAILABLE indicates the corresponding grpc service is not available
    UNAVAILABLE = 2
    # CONNECTING indicates the corresponding protocol is in the connecting phase
    # on this device
    CONNECTING = 3


class ObjectType(betterproto.Enum):
    UNSPECIFIED = 0
    ENTITY = 1
    RELATION = 2
    KIND = 3


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """Event is a topo operation event"""

    type: "EventType" = betterproto.enum_field(1)
    object: "Object" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CreateRequest(betterproto.Message):
    object: "Object" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CreateResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateRequest(betterproto.Message):
    object: "Object" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteRequest(betterproto.Message):
    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListResponse(betterproto.Message):
    objects: List["Object"] = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchRequest(betterproto.Message):
    noreplay: bool = betterproto.bool_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchResponse(betterproto.Message):
    event: "Event" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Object(betterproto.Message):
    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    type: "ObjectType" = betterproto.enum_field(3)
    entity: "Entity" = betterproto.message_field(4, group="obj")
    relation: "Relation" = betterproto.message_field(5, group="obj")
    kind: "Kind" = betterproto.message_field(6, group="obj")
    attributes: Dict[str, str] = betterproto.map_field(
        7, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Entity(betterproto.Message):
    """Entity represents any "thing" that is represented in the topology"""

    # user-defined entity kind
    kind_id: str = betterproto.string_field(1)
    protocols: List["ProtocolState"] = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Relation(betterproto.Message):
    # user defined relation kind
    kind_id: str = betterproto.string_field(1)
    src_entity_id: str = betterproto.string_field(2)
    tgt_entity_id: str = betterproto.string_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Kind(betterproto.Message):
    name: str = betterproto.string_field(1)
    # Map of attributes and their default values for this Kind
    attributes: Dict[str, str] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ProtocolState(betterproto.Message):
    """
    ProtocolState contains information related to service and connectivity to a
    device
    """

    # The protocol to which state relates
    protocol: "Protocol" = betterproto.enum_field(1)
    # ConnectivityState contains the L3 connectivity information
    connectivity_state: "ConnectivityState" = betterproto.enum_field(2)
    # ChannelState relates to the availability of the gRPC channel
    channel_state: "ChannelState" = betterproto.enum_field(3)
    # ServiceState indicates the availability of the gRPC servic on top of the
    # channel
    service_state: "ServiceState" = betterproto.enum_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


class TopoStub(betterproto.ServiceStub):
    """EntityService provides an API for managing entities."""

    async def create(self, *, object: "Object" = None) -> "CreateResponse":
        """Create a new topology object"""

        request = CreateRequest()
        if object is not None:
            request.object = object

        return await self._unary_unary(
            "/onos.topo.Topo/Create", request, CreateResponse
        )

    async def get(self, *, id: str = "") -> "GetResponse":
        """Get an object from topology"""

        request = GetRequest()
        request.id = id

        return await self._unary_unary("/onos.topo.Topo/Get", request, GetResponse)

    async def update(self, *, object: "Object" = None) -> "UpdateResponse":
        """Update an existing topology object"""

        request = UpdateRequest()
        if object is not None:
            request.object = object

        return await self._unary_unary(
            "/onos.topo.Topo/Update", request, UpdateResponse
        )

    async def delete(self, *, id: str = "") -> "DeleteResponse":
        """Delete an object from topology"""

        request = DeleteRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.topo.Topo/Delete", request, DeleteResponse
        )

    async def list(self) -> "ListResponse":
        """List gets a stream of requested objects"""

        request = ListRequest()

        return await self._unary_unary("/onos.topo.Topo/List", request, ListResponse)

    async def watch(self, *, noreplay: bool = False) -> AsyncIterator["WatchResponse"]:
        """Watch returns a stream of topo change notifications"""

        request = WatchRequest()
        request.noreplay = noreplay

        async for response in self._unary_stream(
            "/onos.topo.Topo/Watch",
            request,
            WatchResponse,
        ):
            yield response



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2sub/endpoint/endpoint.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, List, Optional

import betterproto
import grpclib


class EventType(betterproto.Enum):
    """Type of change"""

    NONE = 0
    ADDED = 1
    REMOVED = 3


@dataclass(eq=False, repr=False)
class TerminationEndpoint(betterproto.Message):
    """
    Termination is a record identifying the IP address and TCP port coordinates
    where the E2 termination service is available.
    """

    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    ip: str = betterproto.string_field(3)
    port: int = betterproto.uint32_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """Event is an end-point event"""

    type: "EventType" = betterproto.enum_field(1)
    endpoint: "TerminationEndpoint" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class AddTerminationRequest(betterproto.Message):
    """
    AddTerminationRequest is a request for adding a new termination point
    """

    endpoint: "TerminationEndpoint" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class AddTerminationResponse(betterproto.Message):
    """
    AddTerminationResponse is a response to adding a new termination point
    """

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetTerminationRequest(betterproto.Message):
    """
    GetTerminationRequest is a request for getting existing termination point
    """

    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetTerminationResponse(betterproto.Message):
    """
    GetTerminationResponse is a response with invormation about a requested
    termination point
    """

    endpoint: "TerminationEndpoint" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class RemoveTerminationRequest(betterproto.Message):
    """RemoveTerminationRequest is a request for removing termination point"""

    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class RemoveTerminationResponse(betterproto.Message):
    """
    RemoveTerminationResponse is a response to removing a termination point
    """

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListTerminationsRequest(betterproto.Message):
    """
    ListTerminationsRequest is a request to list all available E2 terminations
    """

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListTerminationsResponse(betterproto.Message):
    """
    ListTerminationsResponse is a response to list all available E2
    terminations
    """

    endpoints: List["TerminationEndpoint"] = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchTerminationsRequest(betterproto.Message):
    """
    WatchTerminationsRequest is a request to receive a stream of all E2
    termination changes.
    """

    noreplay: bool = betterproto.bool_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchTerminationsResponse(betterproto.Message):
    """
    WatchTerminationsResponse is a response indicating a change in the
    available E2 termination end-points.
    """

    event: "Event" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


class E2RegistryServiceStub(betterproto.ServiceStub):
    """
    E2RegistryService manages subscription and subscription delete requests
    """

    async def add_termination(
        self, *, endpoint: "TerminationEndpoint" = None
    ) -> "AddTerminationResponse":
        """AddTermination registers new E2 termination end-point."""

        request = AddTerminationRequest()
        if endpoint is not None:
            request.endpoint = endpoint

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/AddTermination",
            request,
            AddTerminationResponse,
        )

    async def get_termination(self, *, id: str = "") -> "GetTerminationResponse":
        """GetTermination retrieves information about a specific end-point"""

        request = GetTerminationRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/GetTermination",
            request,
            GetTerminationResponse,
        )

    async def remove_termination(self, *, id: str = "") -> "RemoveTerminationResponse":
        """
        RemoveTermination removes the specified E2 termination end-point.
        """

        request = RemoveTerminationRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/RemoveTermination",
            request,
            RemoveTerminationResponse,
        )

    async def list_terminations(self) -> "ListTerminationsResponse":
        """
        ListTerminations returns the list of currently registered E2
        terminations.
        """

        request = ListTerminationsRequest()

        return await self._unary_unary(
            "/onos.e2sub.endpoint.E2RegistryService/ListTerminations",
            request,
            ListTerminationsResponse,
        )

    async def watch_terminations(
        self, *, noreplay: bool = False
    ) -> AsyncIterator["WatchTerminationsResponse"]:
        """
        WatchTerminations returns a stream of changes in the set of available
        E2 terminations.
        """

        request = WatchTerminationsRequest()
        request.noreplay = noreplay

        async for response in self._unary_stream(
            "/onos.e2sub.endpoint.E2RegistryService/WatchTerminations",
            request,
            WatchTerminationsResponse,
        ):
            yield response



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2sub/subscription/subscription.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, List, Optional

import betterproto
import grpclib


class Status(betterproto.Enum):
    """Status is a subscription status"""

    ACTIVE = 0
    PENDING_DELETE = 1
    FAILED = 2


class EventType(betterproto.Enum):
    """EventType is a subscription event type"""

    NONE = 0
    ADDED = 1
    UPDATED = 2
    REMOVED = 3


class Encoding(betterproto.Enum):
    """Encoding indicates a payload encoding"""

    ENCODING_ASN1 = 0
    ENCODING_PROTO = 1


class ActionType(betterproto.Enum):
    ACTION_TYPE_REPORT = 0
    ACTION_TYPE_INSERT = 1
    ACTION_TYPE_POLICY = 2


class SubsequentActionType(betterproto.Enum):
    SUBSEQUENT_ACTION_TYPE_CONTINUE = 0
    SUBSEQUENT_ACTION_TYPE_WAIT = 1


class TimeToWait(betterproto.Enum):
    TIME_TO_WAIT_ZERO = 0
    TIME_TO_WAIT_W1MS = 1
    TIME_TO_WAIT_W2MS = 2
    TIME_TO_WAIT_W5MS = 3
    TIME_TO_WAIT_W10MS = 4
    TIME_TO_WAIT_W20MS = 5
    TIME_TO_WAIT_W30MS = 6
    TIME_TO_WAIT_W40MS = 7
    TIME_TO_WAIT_W50MS = 8
    TIME_TO_WAIT_W100MS = 9
    TIME_TO_WAIT_W200MS = 10
    TIME_TO_WAIT_W500MS = 11
    TIME_TO_WAIT_W1S = 12
    TIME_TO_WAIT_W2S = 13
    TIME_TO_WAIT_W5S = 14
    TIME_TO_WAIT_W10S = 15
    TIME_TO_WAIT_W20S = 16
    TIME_TO_WAIT_W60S = 17


@dataclass(eq=False, repr=False)
class Lifecycle(betterproto.Message):
    """Lifecycle is the subscription lifecycle"""

    status: "Status" = betterproto.enum_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """Event is a subscription event"""

    type: "EventType" = betterproto.enum_field(1)
    subscription: "Subscription" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ServiceModel(betterproto.Message):
    """ServiceModel is a service model definition"""

    id: str = betterproto.string_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Subscription(betterproto.Message):
    """Subscription is a subscription state"""

    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    app_id: str = betterproto.string_field(3)
    details: "SubscriptionDetails" = betterproto.message_field(4)
    lifecycle: "Lifecycle" = betterproto.message_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubscriptionDetails(betterproto.Message):
    e2_node_id: str = betterproto.string_field(1)
    service_model: "ServiceModel" = betterproto.message_field(2)
    event_trigger: "EventTrigger" = betterproto.message_field(3)
    actions: List["Action"] = betterproto.message_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Payload(betterproto.Message):
    encoding: "Encoding" = betterproto.enum_field(1)
    data: bytes = betterproto.bytes_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class EventTrigger(betterproto.Message):
    payload: "Payload" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Action(betterproto.Message):
    id: int = betterproto.int32_field(1)
    type: "ActionType" = betterproto.enum_field(2)
    payload: "Payload" = betterproto.message_field(3)
    subsequent_action: "SubsequentAction" = betterproto.message_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubsequentAction(betterproto.Message):
    """sequence from e2ap-v01.00.00.asn1:1132"""

    type: "SubsequentActionType" = betterproto.enum_field(1)
    time_to_wait: "TimeToWait" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class AddSubscriptionRequest(betterproto.Message):
    """AddSubscriptionRequest a subscription request"""

    subscription: "Subscription" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class AddSubscriptionResponse(betterproto.Message):
    """AddSubscriptionResponse a subscription response"""

    subscription: "Subscription" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class RemoveSubscriptionRequest(betterproto.Message):
    """RemoveSubscriptionRequest a subscription delete request"""

    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class RemoveSubscriptionResponse(betterproto.Message):
    """RemoveSubscriptionResponse a subscription delete response"""

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetSubscriptionRequest(betterproto.Message):
    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetSubscriptionResponse(betterproto.Message):
    subscription: "Subscription" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListSubscriptionsRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListSubscriptionsResponse(betterproto.Message):
    subscriptions: List["Subscription"] = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchSubscriptionsRequest(betterproto.Message):
    noreplay: bool = betterproto.bool_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchSubscriptionsResponse(betterproto.Message):
    event: "Event" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


class E2SubscriptionServiceStub(betterproto.ServiceStub):
    """
    SubscriptionService manages subscription and subscription delete requests
    """

    async def add_subscription(
        self, *, subscription: "Subscription" = None
    ) -> "AddSubscriptionResponse":
        """AddSubscription establishes E2 subscriptions on E2 Node."""

        request = AddSubscriptionRequest()
        if subscription is not None:
            request.subscription = subscription

        return await self._unary_unary(
            "/onos.e2sub.subscription.E2SubscriptionService/AddSubscription",
            request,
            AddSubscriptionResponse,
        )

    async def remove_subscription(
        self, *, id: str = ""
    ) -> "RemoveSubscriptionResponse":
        """RemoveSubscription removes E2 subscriptions on E2 Node."""

        request = RemoveSubscriptionRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.e2sub.subscription.E2SubscriptionService/RemoveSubscription",
            request,
            RemoveSubscriptionResponse,
        )

    async def get_subscription(self, *, id: str = "") -> "GetSubscriptionResponse":
        """
        GetSubscription retrieves information about a specific subscription in
        the list of existing subscriptions
        """

        request = GetSubscriptionRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.e2sub.subscription.E2SubscriptionService/GetSubscription",
            request,
            GetSubscriptionResponse,
        )

    async def list_subscriptions(self) -> "ListSubscriptionsResponse":
        """
        ListSubscriptions returns the list of current existing subscriptions
        """

        request = ListSubscriptionsRequest()

        return await self._unary_unary(
            "/onos.e2sub.subscription.E2SubscriptionService/ListSubscriptions",
            request,
            ListSubscriptionsResponse,
        )

    async def watch_subscriptions(
        self, *, noreplay: bool = False
    ) -> AsyncIterator["WatchSubscriptionsResponse"]:
        """WatchSubscriptions returns a stream of subscription changes"""

        request = WatchSubscriptionsRequest()
        request.noreplay = noreplay

        async for response in self._unary_stream(
            "/onos.e2sub.subscription.E2SubscriptionService/WatchSubscriptions",
            request,
            WatchSubscriptionsResponse,
        ):
            yield response



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/change/types.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


class Phase(betterproto.Enum):
    """Phase is the phase of a NetworkChange"""

    # CHANGE indicates the change has been requested
    CHANGE = 0
    # ROLLBACK indicates a rollback has been requested for the change
    ROLLBACK = 1


class State(betterproto.Enum):
    """State is the state of a phase"""

    # PENDING indicates the phase is pending
    PENDING = 0
    # COMPLETE indicates the phase is complete
    COMPLETE = 2
    # FAILED indicates the phase failed
    FAILED = 3


class Reason(betterproto.Enum):
    """Reason is a reason for a FAILED state"""

    # NONE indicates no error has occurred
    NONE = 0
    # ERROR indicates an error occurred when applying the change
    ERROR = 1


@dataclass(eq=False, repr=False)
class Status(betterproto.Message):
    """Status is the status of a NetworkChange"""

    # 'phase' is the current phase of the NetworkChange
    phase: "Phase" = betterproto.enum_field(1)
    # 'state' is the state of the change within a Phase
    state: "State" = betterproto.enum_field(2)
    # 'reason' is a failure reason
    reason: "Reason" = betterproto.enum_field(3)
    # message is a result message
    message: str = betterproto.string_field(4)
    # incarnation is the status incarnation number
    incarnation: int = betterproto.uint64_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()



#!/usr/bin/env python3
# Copyright 2004-present Facebook. All Rights Reserved.

import collections.abc
import copy
from typing import Dict


def deep_update(dst: Dict, src: Dict, join_lists: bool = False) -> None:
    """Perform an in-place deep merge of ``src`` into ``dst``.

    Args:
        dst: The destination dictionary.
        src: The source dictionary.
        join_lists: Flag to join lists instead of overwriting them.

    Example:
        >>> src = {'foo': {'bar': 'baz'}}
        >>> dst = {'foo': {'bar': 'qux', 'quux': 'quuz'}}
        >>> deep_update(dst, src)
        >>> dst
        {'foo': {'bar': 'baz', 'quux': 'quuz'}}
        >>> src = {"foo": {"bar": ["baz"]}}
        >>> dst = {"foo": {"bar": ["qux"]}}
        >>> deep_update(dst, src, join_lists=True)
        >>> dst
        {"foo": {"bar": ["qux", "baz"]}}
    """
    for k, v in src.items():
        if k in dst:
            if isinstance(v, dict) and isinstance(dst[k], collections.abc.Mapping):
                deep_update(dst[k], v, join_lists)
            elif join_lists and isinstance(v, list) and isinstance(dst[k], list):
                dst[k] = dst[k] + v
            else:
                dst[k] = copy.deepcopy(v)
        else:
            dst[k] = copy.deepcopy(v)



#!/usr/bin/env python3
# Copyright 2004-present Facebook. All Rights Reserved.

import asyncio
import logging
import ssl
from contextlib import suppress
from typing import Dict, List, Optional
from uuid import uuid4

from betterproto.grpc.util.async_channel import AsyncChannel
from grpclib.client import Channel

from .onos_api.onos.e2sub.endpoint import E2RegistryServiceStub, TerminationEndpoint
from .onos_api.onos.e2sub.subscription import (
    Action,
    ActionType,
    E2SubscriptionServiceStub,
    Encoding,
    EventTrigger,
    Payload,
    ServiceModel,
    Subscription,
    SubscriptionDetails,
    SubsequentAction,
    SubsequentActionType,
    TimeToWait,
)
from .onos_api.onos.e2sub.task import E2SubscriptionTaskServiceStub, EventType
from .onos_api.onos.e2t.admin import E2TAdminServiceStub
from .onos_api.onos.e2t.e2 import (
    ControlAckRequest,
    ControlResponse,
    E2TServiceStub,
    EncodingType,
    RequestHeader,
    ServiceModel as E2TServiceModel,
    StreamRequest,
)


class TaskEvent:
    def __init__(self, app_id: str, subscription_id: str, queue: asyncio.Queue) -> None:
        self.app_id = app_id
        self.subscription_id = subscription_id
        self.queue = queue
        self.task: Optional[asyncio.Task] = None
        self.prev_stream: Optional[asyncio.Task] = None

    async def process(self, e2sub_channel: Channel) -> None:
        endpoint_client = E2RegistryServiceStub(e2sub_channel)
        task_client = E2SubscriptionTaskServiceStub(e2sub_channel)

        prev_endpoint_id = None
        async for response in task_client.watch_subscription_tasks(
            subscription_id=self.subscription_id
        ):
            event = response.event
            if event.task.subscription_id != self.subscription_id:
                # Only interested in tasks related to this subscription
                continue
            if event.task.endpoint_id == prev_endpoint_id:
                # Skip if the stream is already open for the associated E2 endpoint
                continue

            if event.type == EventType.NONE or event.type == EventType.CREATED:
                if self.prev_stream is not None:
                    self.prev_stream.cancel()

                prev_endpoint_id = event.task.endpoint_id
                termination = await endpoint_client.get_termination(id=prev_endpoint_id)
                self.prev_stream = asyncio.create_task(
                    self.stream(termination.endpoint)
                )
            elif event.type == EventType.REMOVED:
                prev_endpoint_id = None
                if self.prev_stream is not None:
                    self.prev_stream.cancel()
                    self.prev_stream = None

    async def stream(self, endpoint: TerminationEndpoint) -> None:
        try:
            async with Channel(
                endpoint.ip, endpoint.port, ssl=E2._ssl_context
            ) as channel:
                e2t_client = E2TServiceStub(channel)
                requests: AsyncChannel = AsyncChannel()
                await requests.send_from(
                    [
                        StreamRequest(
                            app_id=self.app_id,
                            subscription_id=self.subscription_id,
                        )
                    ],
                    close=True,
                )
                async for response in e2t_client.stream(requests):
                    await self.queue.put(response.indication_message)
        except Exception:
            logging.exception(f"E2T stream on '{endpoint.ip}:{endpoint.port}' failed")

    async def cancel(self) -> bool:
        if self.task is None or not self.task.cancel():
            return False
        with suppress(asyncio.CancelledError):
            await self.task
            self.task = None

        if self.prev_stream is not None:
            self.prev_stream.cancel()
            with suppress(asyncio.CancelledError):
                await self.prev_stream
                self.prev_stream = None

        return True


class E2:
    _app_id: Optional[str] = None
    _e2t_endpoint: Optional[str] = None
    _e2sub_endpoint: Optional[str] = None
    _e2t_channel: Optional[Channel] = None
    _e2sub_channel: Optional[Channel] = None
    _ssl_context: Optional[ssl.SSLContext] = None
    _subscriptions: Dict[str, TaskEvent] = {}

    @classmethod
    def configure(
        cls,
        app_id: str,
        e2t_endpoint: str,
        e2sub_endpoint: str,
        ca_path: Optional[str] = None,
        cert_path: Optional[str] = None,
        key_path: Optional[str] = None,
        skip_verify: bool = True,
    ) -> None:
        cls._app_id = app_id
        cls._e2t_endpoint = e2t_endpoint
        cls._e2sub_endpoint = e2sub_endpoint

        if ca_path is not None and cert_path is not None and key_path is not None:
            cls._ssl_context = ssl.create_default_context(
                ssl.Purpose.SERVER_AUTH, cafile=ca_path
            )
            cls._ssl_context.load_cert_chain(certfile=cert_path, keyfile=key_path)
            cls._ssl_context.check_hostname = not skip_verify

    @classmethod
    async def start(cls) -> None:
        if cls._e2t_endpoint is None or cls._e2sub_endpoint is None:
            raise Exception("E2 client has not been configured")

        e2t_ip, e2t_port = cls._e2t_endpoint.rsplit(":", 1)
        e2sub_ip, e2sub_port = cls._e2sub_endpoint.rsplit(":", 1)
        cls._e2t_channel = Channel(e2t_ip, int(e2t_port), ssl=cls._ssl_context)
        cls._e2sub_channel = Channel(e2sub_ip, int(e2sub_port), ssl=cls._ssl_context)

    @classmethod
    async def stop(cls) -> None:
        if cls._e2t_channel is None or cls._e2sub_channel is None:
            raise Exception("E2 client has not been started")

        cls._e2t_channel.close()
        cls._e2sub_channel.close()
        cls._e2t_channel = None
        cls._e2sub_channel = None

    @classmethod
    async def list_nodes(cls) -> List[str]:
        if cls._e2t_channel is None:
            raise Exception("E2 client has not been started")

        admin_client = E2TAdminServiceStub(cls._e2t_channel)
        return [conn.id async for conn in admin_client.list_e2_node_connections()]

    @classmethod
    async def send_control(
        cls,
        e2_node_id: str,
        service_model_id: str,
        header: bytes,
        message: bytes,
        ack_request: ControlAckRequest = ControlAckRequest.ACK,
    ) -> ControlResponse:
        if cls._e2t_channel is None:
            raise Exception("E2 client has not been started")

        e2t_client = E2TServiceStub(cls._e2t_channel)
        request_header = RequestHeader(
            encoding_type=EncodingType.PROTO,
            service_model=E2TServiceModel(id=service_model_id),
        )
        return await e2t_client.control(
            header=request_header,
            e2_node_id=e2_node_id,
            control_header=header,
            control_message=message,
            control_ack_request=ack_request,
        )

    @classmethod
    async def subscribe(
        cls,
        e2_node_id: str,
        service_model_id: str,
        trigger: bytes,
        queue: asyncio.Queue,
    ) -> str:
        if cls._e2sub_channel is None or cls._app_id is None:
            raise Exception("E2 client has not been started")

        subscription_client = E2SubscriptionServiceStub(cls._e2sub_channel)
        subscription = Subscription(
            id=str(uuid4()),
            app_id=cls._app_id,
            details=SubscriptionDetails(
                e2_node_id=e2_node_id,
                service_model=ServiceModel(id=service_model_id),
                event_trigger=EventTrigger(
                    payload=Payload(encoding=Encoding.ENCODING_PROTO, data=trigger)
                ),
                actions=[
                    Action(
                        id=10,  # unique action ID for this subscription ID
                        type=ActionType.ACTION_TYPE_REPORT,
                        subsequent_action=SubsequentAction(
                            type=SubsequentActionType.SUBSEQUENT_ACTION_TYPE_CONTINUE,
                            time_to_wait=TimeToWait.TIME_TO_WAIT_ZERO,
                        ),
                    )
                ],
            ),
        )

        await subscription_client.add_subscription(subscription=subscription)
        task_event = TaskEvent(cls._app_id, subscription.id, queue)
        cls._subscriptions[subscription.id] = task_event
        task_event.task = asyncio.create_task(task_event.process(cls._e2sub_channel))
        return subscription.id

    @classmethod
    async def unsubscribe(cls, subscription_id: str) -> bool:
        if cls._e2sub_channel is None:
            raise Exception("E2 client has not been started")
        if subscription_id not in cls._subscriptions:
            return False
        subscription = cls._subscriptions[subscription_id]
        if not subscription.cancel():
            return False

        subscription_client = E2SubscriptionServiceStub(cls._e2sub_channel)
        try:
            await subscription_client.remove_subscription(id=subscription_id)
            del cls._subscriptions[subscription_id]
            return True
        except Exception:
            logging.exception(f"Failed to unsubscribe on {subscription_id}")
            return False



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2sub/task/task.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, List, Optional

import betterproto
import grpclib


class Phase(betterproto.Enum):
    """Phase is a subscription task phase"""

    # OPEN is a subscription task open phase
    OPEN = 0
    # CLOSE is a subscription task close phase
    CLOSE = 1


class Status(betterproto.Enum):
    """Status is a subscription task status"""

    # PENDING indicates the subscription task phase is pending
    PENDING = 0
    # COMPLETE indicates the subscription task phase is complete
    COMPLETE = 1
    # FAILED indicates the subscription task phase failed
    FAILED = 2


class Cause(betterproto.Enum):
    """Cause is a failure cause"""

    CAUSE_UNKNOWN = 0
    CAUSE_MISC_CONTROL_PROCESSING_OVERLOAD = 1
    CAUSE_MISC_HARDWARE_FAILURE = 2
    CAUSE_MISC_OM_INTERVENTION = 3
    CAUSE_MISC_UNSPECIFIED = 4
    CAUSE_PROTOCOL_TRANSFER_SYNTAX_ERROR = 5
    CAUSE_PROTOCOL_ABSTRACT_SYNTAX_ERROR_REJECT = 6
    CAUSE_PROTOCOL_ABSTRACT_SYNTAX_ERROR_IGNORE_AND_NOTIFY = 7
    CAUSE_PROTOCOL_MESSAGE_NOT_COMPATIBLE_WITH_RECEIVER_STATE = 8
    CAUSE_PROTOCOL_SEMANTIC_ERROR = 9
    CAUSE_PROTOCOL_ABSTRACT_SYNTAX_ERROR_FALSELY_CONSTRUCTED_MESSAGE = 10
    CAUSE_PROTOCOL_UNSPECIFIED = 11
    CAUSE_RIC_RAN_FUNCTION_ID_INVALID = 12
    CAUSE_RIC_ACTION_NOT_SUPPORTED = 13
    CAUSE_RIC_EXCESSIVE_ACTIONS = 14
    CAUSE_RIC_DUPLICATE_ACTION = 15
    CAUSE_RIC_DUPLICATE_EVENT = 16
    CAUSE_RIC_FUNCTION_RESOURCE_LIMIT = 17
    CAUSE_RIC_REQUEST_ID_UNKNOWN = 18
    CAUSE_RIC_INCONSISTENT_ACTION_SUBSEQUENT_ACTION_SEQUENCE = 19
    CAUSE_RIC_CONTROL_MESSAGE_INVALID = 20
    CAUSE_RIC_CALL_PROCESS_ID_INVALID = 21
    CAUSE_RIC_UNSPECIFIED = 22
    CAUSE_RICSERVICE_FUNCTION_NOT_REQUIRED = 23
    CAUSE_RICSERVICE_EXCESSIVE_FUNCTIONS = 24
    CAUSE_RICSERVICE_RIC_RESOURCE_LIMIT = 25
    CAUSE_TRANSPORT_UNSPECIFIED = 26
    CAUSE_TRANSPORT_TRANSPORT_RESOURCE_UNAVAILABLE = 27


class EventType(betterproto.Enum):
    """Type of change"""

    NONE = 0
    CREATED = 1
    UPDATED = 2
    REMOVED = 3


@dataclass(eq=False, repr=False)
class Lifecycle(betterproto.Message):
    """Lifecycle is a subscription task status"""

    phase: "Phase" = betterproto.enum_field(1)
    status: "Status" = betterproto.enum_field(2)
    failure: "Failure" = betterproto.message_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Failure(betterproto.Message):
    """Failure is a subscription failure status"""

    cause: "Cause" = betterproto.enum_field(1)
    message: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class SubscriptionTask(betterproto.Message):
    """
    SubscriptionTask is a task representing a subscription between an E2
    termination and an E2 node
    """

    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    subscription_id: str = betterproto.string_field(3)
    endpoint_id: str = betterproto.string_field(4)
    lifecycle: "Lifecycle" = betterproto.message_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """Event is a SubscriptionTask event"""

    type: "EventType" = betterproto.enum_field(1)
    task: "SubscriptionTask" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetSubscriptionTaskRequest(betterproto.Message):
    """
    GetSubscriptionTaskRequest is a request for getting existing
    SubscriptionTask
    """

    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetSubscriptionTaskResponse(betterproto.Message):
    """
    GetSubscriptionTaskResponse is a response with invormation about a
    requested SubscriptionTask
    """

    task: "SubscriptionTask" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListSubscriptionTasksRequest(betterproto.Message):
    """
    ListSubscriptionTasksRequest is a request to list all available
    SubscriptionTasks
    """

    subscription_id: str = betterproto.string_field(1)
    endpoint_id: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListSubscriptionTasksResponse(betterproto.Message):
    """
    ListSubscriptionTasksResponse is a response to list all available
    SubscriptionTasks
    """

    tasks: List["SubscriptionTask"] = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchSubscriptionTasksRequest(betterproto.Message):
    """
    WatchSubscriptionTasksRequest is a request to receive a stream of all
    SubscriptionTask changes.
    """

    noreplay: bool = betterproto.bool_field(1)
    subscription_id: str = betterproto.string_field(2)
    endpoint_id: str = betterproto.string_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class WatchSubscriptionTasksResponse(betterproto.Message):
    """
    WatchSubscriptionTasksResponse is a response indicating a change in the
    available SubscriptionTasks.
    """

    event: "Event" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateSubscriptionTaskRequest(betterproto.Message):
    """
    UpdateSubscriptionTaskRequest is a request for updating a SubscriptionTask
    status
    """

    task: "SubscriptionTask" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UpdateSubscriptionTaskResponse(betterproto.Message):
    """
    UpdateSubscriptionTaskResponse is a response to updating a SubscriptionTask
    status
    """

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


class E2SubscriptionTaskServiceStub(betterproto.ServiceStub):
    """
    E2SubscriptionTaskService manages subscription tasks between E2 termination
    points and E2 nodes
    """

    async def get_subscription_task(
        self, *, id: str = ""
    ) -> "GetSubscriptionTaskResponse":
        """GetSubscriptionTask retrieves information about a specific task"""

        request = GetSubscriptionTaskRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.e2sub.task.E2SubscriptionTaskService/GetSubscriptionTask",
            request,
            GetSubscriptionTaskResponse,
        )

    async def list_subscription_tasks(
        self, *, subscription_id: str = "", endpoint_id: str = ""
    ) -> "ListSubscriptionTasksResponse":
        """
        ListSubscriptionTasks returns the list of currently registered E2
        Tasks.
        """

        request = ListSubscriptionTasksRequest()
        request.subscription_id = subscription_id
        request.endpoint_id = endpoint_id

        return await self._unary_unary(
            "/onos.e2sub.task.E2SubscriptionTaskService/ListSubscriptionTasks",
            request,
            ListSubscriptionTasksResponse,
        )

    async def watch_subscription_tasks(
        self,
        *,
        noreplay: bool = False,
        subscription_id: str = "",
        endpoint_id: str = "",
    ) -> AsyncIterator["WatchSubscriptionTasksResponse"]:
        """
        WatchSubscriptionTasks returns a stream of changes in the set of
        available E2 Tasks.
        """

        request = WatchSubscriptionTasksRequest()
        request.noreplay = noreplay
        request.subscription_id = subscription_id
        request.endpoint_id = endpoint_id

        async for response in self._unary_stream(
            "/onos.e2sub.task.E2SubscriptionTaskService/WatchSubscriptionTasks",
            request,
            WatchSubscriptionTasksResponse,
        ):
            yield response

    async def update_subscription_task(
        self, *, task: "SubscriptionTask" = None
    ) -> "UpdateSubscriptionTaskResponse":
        """UpdateSubscriptionTask updates a task status"""

        request = UpdateSubscriptionTaskRequest()
        if task is not None:
            request.task = task

        return await self._unary_unary(
            "/onos.e2sub.task.E2SubscriptionTaskService/UpdateSubscriptionTask",
            request,
            UpdateSubscriptionTaskResponse,
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/configmodel/registry.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List, Optional

import betterproto
import grpclib


@dataclass(eq=False, repr=False)
class ConfigModel(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)
    modules: List["ConfigModule"] = betterproto.message_field(3)
    files: Dict[str, str] = betterproto.map_field(
        4, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ConfigModule(betterproto.Message):
    name: str = betterproto.string_field(1)
    file: str = betterproto.string_field(2)
    revision: str = betterproto.string_field(3)
    organization: str = betterproto.string_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetModelRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetModelResponse(betterproto.Message):
    model: "ConfigModel" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListModelsRequest(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListModelsResponse(betterproto.Message):
    models: List["ConfigModel"] = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class PushModelRequest(betterproto.Message):
    model: "ConfigModel" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class PushModelResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteModelRequest(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class DeleteModelResponse(betterproto.Message):
    pass

    def __post_init__(self) -> None:
        super().__post_init__()


class ConfigModelRegistryServiceStub(betterproto.ServiceStub):
    async def get_model(
        self, *, name: str = "", version: str = ""
    ) -> "GetModelResponse":

        request = GetModelRequest()
        request.name = name
        request.version = version

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/GetModel",
            request,
            GetModelResponse,
        )

    async def list_models(self) -> "ListModelsResponse":

        request = ListModelsRequest()

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/ListModels",
            request,
            ListModelsResponse,
        )

    async def push_model(self, *, model: "ConfigModel" = None) -> "PushModelResponse":

        request = PushModelRequest()
        if model is not None:
            request.model = model

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/PushModel",
            request,
            PushModelResponse,
        )

    async def delete_model(
        self, *, name: str = "", version: str = ""
    ) -> "DeleteModelResponse":

        request = DeleteModelRequest()
        request.name = name
        request.version = version

        return await self._unary_unary(
            "/onos.configmodel.ConfigModelRegistryService/DeleteModel",
            request,
            DeleteModelResponse,
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/snapshot/device/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class DeviceSnapshot(betterproto.Message):
    """DeviceSnapshot is a device snapshot"""

    # 'id' is the unique snapshot identifier
    id: str = betterproto.string_field(1)
    # 'device_id' is the device to which the snapshot applies
    device_id: str = betterproto.string_field(2)
    # 'device_version' is the version to which the snapshot applies
    device_version: str = betterproto.string_field(3)
    # 'device_type' is an optional device type to which to apply this change
    device_type: str = betterproto.string_field(4)
    # 'revision' is the request revision number
    revision: int = betterproto.uint64_field(5)
    # 'network_snapshot' is a reference to the network snapshot from which this
    # snapshot was created
    network_snapshot: "NetworkSnapshotRef" = betterproto.message_field(6)
    # 'max_network_change_index' is the maximum network change index to be
    # snapshotted for the device
    max_network_change_index: int = betterproto.uint64_field(7)
    # 'status' is the snapshot status
    status: "__snapshot__.Status" = betterproto.message_field(8)
    # 'created' is the time at which the configuration was created
    created: datetime = betterproto.message_field(9)
    # 'updated' is the time at which the configuration was last updated
    updated: datetime = betterproto.message_field(10)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class NetworkSnapshotRef(betterproto.Message):
    """
    NetworkSnapshotRef is a back reference to the NetworkSnapshot that created
    a DeviceSnapshot
    """

    # 'id' is the identifier of the network snapshot from which this snapshot was
    # created
    id: str = betterproto.string_field(1)
    # 'index' is the index of the network snapshot from which this snapshot was
    # created
    index: int = betterproto.uint64_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Snapshot(betterproto.Message):
    """Snapshot is a snapshot of the state of a single device"""

    # 'id' is a unique snapshot identifier
    id: str = betterproto.string_field(1)
    # 'device_id' is the device to which the snapshot applies
    device_id: str = betterproto.string_field(2)
    # 'device_version' is the version to which the snapshot applies
    device_version: str = betterproto.string_field(3)
    # 'device_type' is an optional device type to which to apply this change
    device_type: str = betterproto.string_field(4)
    # 'snapshot_id' is the ID of the snapshot
    snapshot_id: str = betterproto.string_field(5)
    # 'change_index' is the change index at which the snapshot ended
    change_index: int = betterproto.uint64_field(6)
    # 'values' is a list of values to set
    values: List["__change_device__.PathValue"] = betterproto.message_field(7)

    def __post_init__(self) -> None:
        super().__post_init__()


from ... import snapshot as __snapshot__
from ...change import device as __change_device__



#!/usr/bin/env python3
# Copyright 2004-present Facebook. All Rights Reserved.

import json
import logging
from typing import Callable, cast

from aiohttp import web

from .utils.dict import deep_update


routes = web.RouteTableDef()


@web.middleware
async def error_middleware(request: web.Request, handler: Callable) -> web.Response:
    """Middleware for returning non-200 responses as JSON."""
    try:
        return cast(web.Response, await handler(request))
    except web.HTTPError as e:
        return web.json_response(
            {"status": "error", "message": e.text}, status=e.status
        )
    except Exception:
        logging.exception("Error handling request")
        return web.json_response(
            {"status": "error", "message": "Server got itself in trouble"}, status=500
        )


@routes.get("/status")
async def handle_get_status(request: web.Request) -> web.Response:
    """
    ---
    description: Check if the webserver is responsive.
    tags:
    - Health
    responses:
      "200":
        description: Successful operation.
        content:
          text/plain:
            example: "Alive"
    """
    return web.Response(text="Alive")


@routes.get("/config")
async def handle_get_config(request: web.Request) -> web.Response:
    """
    ---
    description: Return the current service configuration settings.
    tags:
    - Configuration
    responses:
      "200":
        description: Successful operation.
        content:
          application/json:
            example:
              foo:
              - bar
              - baz
              timeout: 15
      "500":
        description: Failed to load or parse the configuration file.
    """
    try:
        with open("./service_config.json") as f:
            config = json.load(f)
            return web.json_response(config)
    except json.JSONDecodeError:
        raise web.HTTPInternalServerError(
            text="Existing configuration is not valid JSON"
        )
    except OSError:
        raise web.HTTPInternalServerError(text="Failed to load existing configuration")


@routes.put("/config")
async def handle_set_config(request: web.Request) -> web.Response:
    """
    ---
    description: Completely overwrite the service's configuration settings.
    tags:
    - Configuration
    requestBody:
      description: New service configuration object
      required: true
      content:
        application/json:
          example:
            config:
              foo:
              - bar
              - baz
              timeout: 30
    responses:
      "200":
        description: Successful operation.
        content:
          application/json:
            example:
              foo:
              - bar
              - baz
              timeout: 30
      "400":
        description: Missing or invalid 'config' parameter.
      "500":
        description: Failed to overwrite service configuration.
    """
    body = await request.json()

    if "config" not in body:
        raise web.HTTPBadRequest(text="Missing required 'config' param")

    config = body["config"]
    if not isinstance(config, dict):
        raise web.HTTPBadRequest(text="Invalid value for 'config': Not object")

    try:
        with open("./service_config.json", "w") as f:
            json.dump(config, f)

        # Trigger the shutdown event
        request.app["shutdown_event"].set()
        return web.json_response(config)
    except OSError:
        raise web.HTTPInternalServerError(text="Failed to overwrite config")


@routes.patch("/config")
async def handle_update_config(request: web.Request) -> web.Response:
    """
    ---
    description: Partially update the service's configuration settings.
    tags:
    - Configuration
    requestBody:
      description: New service configuration object
      required: true
      content:
        application/json:
          example:
            config:
              timeout: 60
    responses:
      "200":
        description: Successful operation.
        content:
          application/json:
            example:
              foo:
              - bar
              - baz
              timeout: 60
      "400":
        description: Missing or invalid 'config' parameter.
      "500":
        description: Failed to update service configuration.
    """
    body = await request.json()

    if "config" not in body:
        raise web.HTTPBadRequest(text="Missing required 'config' param")

    updates = body["config"]
    if not isinstance(updates, dict):
        raise web.HTTPBadRequest(text="Invalid value for 'config': Not object")

    try:
        with open("./service_config.json", "r+") as f:
            config = json.load(f)
            deep_update(config, updates)

            # Write new config at the beginning of the file and truncate what's left
            f.seek(0)
            json.dump(config, f)
            f.truncate()

        # Trigger the shutdown event
        request.app["shutdown_event"].set()
        return web.json_response(config)
    except json.JSONDecodeError:
        raise web.HTTPInternalServerError(
            text="Existing configuration is not valid JSON"
        )
    except OSError:
        raise web.HTTPInternalServerError(text="Failed to update configuration")


@routes.put("/log/{level:[A-Z]+}")
async def handle_set_log_level(request: web.Request) -> web.Response:
    """
    ---
    description: Dynamically set the service's log level.
    tags:
    - Configuration
    parameters:
    - in: path
      name: level
      description: The new log level.
      required: true
      schema:
        enum: [DEBUG, INFO, WARNING, ERROR, FATAL]
        type: string
    responses:
      "200":
        description: Successful operation.
        content:
          text/plain:
            example: "Log level set to WARNING from INFO"
      "400":
        description: Invalid log level.
    """
    level = request.match_info["level"]
    prev_level = logging.getLevelName(logging.root.level)

    if level == prev_level:
        return web.Response(text=f"Log level is already {prev_level}")

    try:
        logging.root.setLevel(level)
    except ValueError as e:
        raise web.HTTPBadRequest(text=str(e))

    return web.Response(text=f"Log level set to {level} from {prev_level}")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/ransim/types/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List

import betterproto


@dataclass(eq=False, repr=False)
class Point(betterproto.Message):
    lat: float = betterproto.double_field(1)
    lng: float = betterproto.double_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Sector(betterproto.Message):
    azimuth: int = betterproto.int32_field(1)
    arc: int = betterproto.int32_field(2)
    centroid: "Point" = betterproto.message_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Route(betterproto.Message):
    name: int = betterproto.uint64_field(1)
    waypoints: List["Point"] = betterproto.message_field(2)
    color: str = betterproto.string_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Ue(betterproto.Message):
    imsi: int = betterproto.uint64_field(1)
    type: str = betterproto.string_field(2)
    position: "Point" = betterproto.message_field(4)
    rotation: int = betterproto.uint32_field(5)
    serving_tower: int = betterproto.uint64_field(7)
    serving_tower_strength: float = betterproto.double_field(8)
    tower1: int = betterproto.uint64_field(9)
    tower1_strength: float = betterproto.double_field(10)
    tower2: int = betterproto.uint64_field(11)
    tower2_strength: float = betterproto.double_field(12)
    tower3: int = betterproto.uint64_field(13)
    tower3_strength: float = betterproto.double_field(14)
    crnti: int = betterproto.uint32_field(15)
    admitted: bool = betterproto.bool_field(16)
    metrics: "UeMetrics" = betterproto.message_field(17)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class UeMetrics(betterproto.Message):
    # Latency (in nanoseconds) of the most recent hand-over
    ho_latency: int = betterproto.int64_field(1)
    # Handover report timestamp (in nanoseconds since epoch)
    ho_report_timestamp: int = betterproto.int64_field(2)
    # flag to indicate the first measurement
    is_first: bool = betterproto.bool_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Cell(betterproto.Message):
    ecgi: int = betterproto.uint64_field(1)
    location: "Point" = betterproto.message_field(3)
    sector: "Sector" = betterproto.message_field(4)
    color: str = betterproto.string_field(5)
    max_ues: int = betterproto.uint32_field(6)
    neighbors: List[int] = betterproto.uint64_field(7)
    # The cell transmit power in decibels
    tx_power_db: float = betterproto.double_field(8)
    # crntis maps a ue's name to its crnti
    crnti_map: Dict[int, int] = betterproto.map_field(
        9, betterproto.TYPE_UINT32, betterproto.TYPE_UINT64
    )
    crnti_index: int = betterproto.uint32_field(10)
    port: int = betterproto.uint32_field(11)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Node(betterproto.Message):
    enbid: int = betterproto.uint32_field(1)
    controllers: List[str] = betterproto.string_field(2)
    service_models: List[str] = betterproto.string_field(3)
    cell_ecgis: List[int] = betterproto.uint64_field(4)
    status: str = betterproto.string_field(5)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class MapLayout(betterproto.Message):
    # Map center latitude and longitude
    center: "Point" = betterproto.message_field(1)
    # The starting Zoom level
    zoom: float = betterproto.float_field(2)
    # Show map as faded on start
    fade: bool = betterproto.bool_field(3)
    # Show routes on start
    show_routes: bool = betterproto.bool_field(4)
    # Show power as circle on start
    show_power: bool = betterproto.bool_field(5)
    # Ratio of random locations diameter to tower grid width
    locations_scale: float = betterproto.float_field(9)
    # FIXME: These are deprecated; remove Max number of UEs for complete
    # simulation
    min_ues: int = betterproto.uint32_field(6)
    # Max number of UEs for complete simulation
    max_ues: int = betterproto.uint32_field(7)
    # the current number of routes
    current_routes: int = betterproto.uint32_field(8)

    def __post_init__(self) -> None:
        super().__post_init__()



#!/usr/bin/env python3
# Copyright 2004-present Facebook. All Rights Reserved.

__all__ = ["init"]

import asyncio
import logging
import os
import signal
from typing import Callable, Optional

from aiohttp import web
from aiohttp_swagger import setup_swagger

from .e2 import E2
from .server import error_middleware, routes


def init(
    main: Callable,
    app: Optional[web.Application] = None,
    add_metrics_route: bool = False,
    use_uvloop: bool = False,
    listen_on_ipv6: bool = False,
) -> None:
    """Start the webserver and the entrypoint logic passed in as ``main``.

    Args:
        main: A ``lambda`` function wrapper to the entrypoint for the service's logic.
        app: An existing web application object, if available.
        add_metrics_route: Adds an endpoint for Prometheus to scrape metrics.
        use_uvloop: Uses the uvloop event loop instead of asyncio's built-in loop.
        listen_on_ipv6: Instructs the webserver to listen on "::" instead of "0.0.0.0".

    Raises:
        RuntimeError: A user-requested runtime dependency is missing.
    """
    logging.basicConfig(
        format="%(levelname)s %(asctime)s %(filename)s:%(lineno)d] %(message)s",
        level=logging.INFO,
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    # Use uvloop to make asyncio fast
    if use_uvloop:
        try:
            import uvloop

            uvloop.install()
        except ImportError:
            raise RuntimeError("uvloop is not installed")

    # Create web application object and shutdown event
    if app is None:
        app = web.Application(middlewares=[error_middleware])
    app["main"] = main
    app["shutdown_event"] = asyncio.Event()

    # Initialize routes for the HTTP server
    app.add_routes(routes)

    # Add a /metrics route for exposing Prometheus metrics
    if add_metrics_route:
        try:
            from prometheus_async import aio

            app.router.add_get("/metrics", aio.web.server_stats)
        except ImportError:
            raise RuntimeError("prometheus_async is not installed")

    # Document HTTP API endpoints with swagger
    setup_swagger(app, ui_version=3)

    # Add background tasks
    app.on_startup.append(_start_background_tasks)
    app.on_cleanup.append(_stop_background_tasks)
    web.run_app(app, host="::" if listen_on_ipv6 else "0.0.0.0")


async def _start_background_tasks(app: web.Application) -> None:
    """Start the E2 and create the main_wrapper and shutdown_listener tasks."""

    await E2.start()
    app["main_wrapper_task"] = asyncio.create_task(_main_wrapper(app))
    app["shutdown_listener_task"] = asyncio.create_task(_shutdown_listener(app))


async def _stop_background_tasks(app: web.Application) -> None:
    """Cancel the shutdown_listener and main_wrapper tasks and stop the E2."""
    try:
        app["shutdown_listener_task"].cancel()
        await app["shutdown_listener_task"]
    except asyncio.CancelledError:
        pass

    if not app["main_wrapper_task"].done():
        try:
            app["main_wrapper_task"].cancel()
            await app["main_wrapper_task"]
        except asyncio.CancelledError:
            pass

    await E2.stop()

    # Raise the exception caught in the main_wrapper if the task wasn't cancelled
    if not app["main_wrapper_task"].cancelled():
        await app["main_wrapper_task"]


async def _main_wrapper(app: web.Application) -> None:
    """Run the supplied 'main' and set the shutdown event if it fails."""
    try:
        await app["main"]()
    except:  # noqa: E722
        app["shutdown_event"].set()
        raise


async def _shutdown_listener(app: web.Application) -> None:
    """Wait for the shutdown_event notification to kill the process."""
    await app["shutdown_event"].wait()
    logging.info("Shutting down!")

    # Sleep for 1 second before terminating
    await asyncio.sleep(1)
    os.kill(os.getpid(), signal.SIGTERM)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/change/device/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import betterproto


class ValueType(betterproto.Enum):
    """ValueType is the type for a value"""

    EMPTY = 0
    STRING = 1
    INT = 2
    UINT = 3
    BOOL = 4
    DECIMAL = 5
    FLOAT = 6
    BYTES = 7
    LEAFLIST_STRING = 8
    LEAFLIST_INT = 9
    LEAFLIST_UINT = 10
    LEAFLIST_BOOL = 11
    LEAFLIST_DECIMAL = 12
    LEAFLIST_FLOAT = 13
    LEAFLIST_BYTES = 14


@dataclass(eq=False, repr=False)
class DeviceChange(betterproto.Message):
    """DeviceChange is a stored configuration change for a single device"""

    # 'id' is the unique identifier of the change
    id: str = betterproto.string_field(1)
    # 'index' is a monotonically increasing, globally unique index of the change
    # The index is provided by the store, is static and unique for each unique
    # change identifier, and should not be modified by client code.
    index: int = betterproto.uint64_field(2)
    # 'revision' is the change revision number The revision number is provided by
    # the store and should not be modified by client code. Each unique state of
    # the change will be assigned a unique revision number which can be used for
    # optimistic concurrency control when updating or deleting the change state.
    revision: int = betterproto.uint64_field(3)
    # 'network_change' is a reference to the NetworkChange that created this
    # change
    network_change: "NetworkChangeRef" = betterproto.message_field(4)
    # 'change' is the change object
    change: "Change" = betterproto.message_field(5)
    # 'status' is the lifecycle status of the change
    status: "__change__.Status" = betterproto.message_field(6)
    # 'created' is the time at which the change was created
    created: datetime = betterproto.message_field(7)
    # 'updated' is the time at which the change was last updated
    updated: datetime = betterproto.message_field(8)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class NetworkChangeRef(betterproto.Message):
    """
    NetworkChangeRef is a back reference to the NetworkChange that created a
    DeviceChange
    """

    # 'id' is the identifier of the network change from which this change was
    # created
    id: str = betterproto.string_field(1)
    # 'index' is the index of the network change from which this change was
    # created
    index: int = betterproto.uint64_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Change(betterproto.Message):
    """Change represents a configuration change to a single device"""

    # 'device_id' is the identifier of the device to which this change applies
    device_id: str = betterproto.string_field(1)
    # 'device_version' is an optional device version to which to apply this
    # change
    device_version: str = betterproto.string_field(2)
    # 'device_type' is an optional device type to which to apply this change
    device_type: str = betterproto.string_field(3)
    # 'values' is a set of change values to apply
    values: List["ChangeValue"] = betterproto.message_field(4)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class TypedValue(betterproto.Message):
    """TypedValue is a value represented as a byte array"""

    # 'bytes' is the bytes array
    bytes: bytes = betterproto.bytes_field(1)
    # 'type' is the value type
    type: "ValueType" = betterproto.enum_field(2)
    # 'type_opts' is a set of type options
    type_opts: List[int] = betterproto.int32_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ChangeValue(betterproto.Message):
    """
    ChangeValue is an individual Path/Value and removed flag combination in a
    Change
    """

    # 'path' is the path to change
    path: str = betterproto.string_field(1)
    # 'value' is the change value
    value: "TypedValue" = betterproto.message_field(2)
    # 'removed' indicates whether this is a delete
    removed: bool = betterproto.bool_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class PathValue(betterproto.Message):
    """
    PathValue is an individual Path/Value combination - it is like ChangeValue
    above without the removed flag - it is not used in the DeviceChange store
    Instead it is useful for handling OpState and Snapshots where `removed` is
    not relevant
    """

    # 'path' is the path to change
    path: str = betterproto.string_field(1)
    # 'value' is the change value
    value: "TypedValue" = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


from ... import change as __change__



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aiomsa',
 'aiomsa.onos_api',
 'aiomsa.onos_api.gnmi',
 'aiomsa.onos_api.gnmi_ext',
 'aiomsa.onos_api.gogoproto',
 'aiomsa.onos_api.onos',
 'aiomsa.onos_api.onos.config',
 'aiomsa.onos_api.onos.config.admin',
 'aiomsa.onos_api.onos.config.change',
 'aiomsa.onos_api.onos.config.change.device',
 'aiomsa.onos_api.onos.config.change.network',
 'aiomsa.onos_api.onos.config.diags',
 'aiomsa.onos_api.onos.config.snapshot',
 'aiomsa.onos_api.onos.config.snapshot.device',
 'aiomsa.onos_api.onos.config.snapshot.network',
 'aiomsa.onos_api.onos.configmodel',
 'aiomsa.onos_api.onos.e2sub',
 'aiomsa.onos_api.onos.e2sub.endpoint',
 'aiomsa.onos_api.onos.e2sub.subscription',
 'aiomsa.onos_api.onos.e2sub.task',
 'aiomsa.onos_api.onos.e2t',
 'aiomsa.onos_api.onos.e2t.admin',
 'aiomsa.onos_api.onos.e2t.e2',
 'aiomsa.onos_api.onos.kpimon',
 'aiomsa.onos_api.onos.ransim',
 'aiomsa.onos_api.onos.ransim.metrics',
 'aiomsa.onos_api.onos.ransim.model',
 'aiomsa.onos_api.onos.ransim.trafficsim',
 'aiomsa.onos_api.onos.ransim.types',
 'aiomsa.onos_api.onos.topo',
 'aiomsa.utils']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp-swagger>=1.0.9,<2.0.0',
 'aiohttp>=3.5.4,<4.0.0',
 'betterproto>=2.0.0b2,<3.0.0']

extras_require = \
{'docs': ['furo>=2021.2.21-beta.25,<2022.0.0',
          'sphinx>=3.4.3,<4.0.0',
          'sphinx-autodoc-typehints>=1.11.1,<2.0.0',
          'sphinxcontrib-openapi>=0.7.0,<0.8.0'],
 'prometheus': ['prometheus_async>=19.2.0,<20.0.0'],
 'speedups': ['uvloop>=0.14.0,<0.15.0']}

setup_kwargs = {
    'name': 'aiomsa',
    'version': '0.1.0a1',
    'description': 'Asynchronous xApp framework',
    'long_description': '# aiomsa\n[![build](https://github.com/facebookexternal/aiomsa/workflows/build/badge.svg)](https://github.com/facebookexternal/aiomsa/actions?query=workflow%3Abuild)\n[![style](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n*aiomsa* is a Python 3.7+ framework built using `asyncio`. At its core, *aiomsa*\nprovides a simple and standardized way to write xApps that can be deployed as\nmicroservices in Python.\n\n## Installation\n*aiomsa* can be installed from PyPI.\n```bash\npip install aiomsa\n```\n\nYou can also get the latest code from GitHub.\n```bash\npoetry add git+https://github.com/facebookexternal/aiomsa\n```\n\n## Getting Started\nThe follwing example shows how to use *aiomsa* to create a simple xApp for subscribing\nto the E2T service for a particular custom service model.\n\n```python\nimport asyncio\nimport json\n\nfrom aiomsa import init\nfrom aiomsa.e2 import E2\n\nfrom .models import MyModel\n\n\nasync def consume(queue):\n   while True:\n      record = await queue.get()\n      print(record)\n\nasync def produce(queue):\n   conns = await E2.list_nodes()\n   subscription_id = await E2.subscribe(\n      conns[0],\n      service_model_id="mymodel_v1",\n      trigger=bytes(MyModel(param="foo")),\n      queue=queue,\n   )\n\nasync def async_main():\n   """Create an asyncio.Queue to pub/sub records."""\n\n   queue = asyncio.Queue()\n   await asyncio.gather(produce(queue), consume(queue))\n\n\nif __name__ == "__main__":\n   with open("./config.json") as f:\n      config = json.load(f)\n\n   E2.configure(**config)\n   init(lambda: async_main())\n```\n',
    'author': 'Facebook Connectivity',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/facebookexternal/aiomsa',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/kpimon/kpimon.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import betterproto
import grpclib


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    id: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Object(betterproto.Message):
    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    attributes: Dict[str, int] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_UINT64
    )

    def __post_init__(self) -> None:
        super().__post_init__()


class KpimonStub(betterproto.ServiceStub):
    async def get_num_active_u_es(self, *, id: str = "") -> "GetResponse":

        request = GetRequest()
        request.id = id

        return await self._unary_unary(
            "/onos.kpimon.Kpimon/GetNumActiveUEs", request, GetResponse
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/admin/admin.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import timedelta
from typing import AsyncIterable, AsyncIterator, Iterable, List, Optional, Union

import betterproto
import grpclib


class Type(betterproto.Enum):
    """Streaming event type"""

    # NONE indicates this response does not represent a state change
    NONE = 0
    # ADDED is an event which occurs when an item is added
    ADDED = 1
    # UPDATED is an event which occurs when an item is updated
    UPDATED = 2
    # REMOVED is an event which occurs when an item is removed
    REMOVED = 3


@dataclass(eq=False, repr=False)
class ReadOnlySubPath(betterproto.Message):
    """
    ReadOnlySubPath is an extension to the ReadOnlyPath to define the datatype
    of the subpath
    """

    # sub_path is the relative path of a child object e.g. /list2b/index
    sub_path: str = betterproto.string_field(1)
    # value_type is the datatype of the read only path
    value_type: "_change_device__.ValueType" = betterproto.enum_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ReadOnlyPath(betterproto.Message):
    """
    ReadOnlyPath extracted from the model plugin as the definition of a tree of
    read only items. In YANG models items are defined as ReadOnly with the
    `config false` keyword. This can be applied to single items (leafs) or
    collections (containers or lists). When this `config false` is applied to
    an object every item beneath it will also become readonly - here these are
    shown as subpaths. The complete read only path then will be a concatenation
    of both e.g. /cont1a/cont1b-state/list2b/index and the type is defined in
    the SubPath as UInt8.
    """

    # path of the topmost `config false` object e.g. /cont1a/cont1b-state
    path: str = betterproto.string_field(1)
    # ReadOnlySubPath is a set of children of the path including an entry for the
    # type of the topmost object with subpath `/` An example is /list2b/index
    sub_path: List["ReadOnlySubPath"] = betterproto.message_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ReadWritePath(betterproto.Message):
    """
    ReadWritePath is extracted from the model plugin as the definition of a
    writeable attributes. In YANG models items are writable by default unless
    they are specified as `config false` or have an item with `config false` as
    a parent. Each configurable item has metadata with meanings taken from the
    YANG specification RFC 6020.
    """

    # path is the full path to the attribute (leaf or leaf-list)
    path: str = betterproto.string_field(1)
    # value_type is the data type of the attribute
    value_type: "_change_device__.ValueType" = betterproto.enum_field(2)
    # units is the unit of measurement e.g. dB, mV
    units: str = betterproto.string_field(3)
    # description is an explaination of the meaning of the attribute
    description: str = betterproto.string_field(4)
    # mandatory shows whether the attribute is optional (false) or required
    # (true)
    mandatory: bool = betterproto.bool_field(5)
    # default is a default value used with optional attributes
    default: str = betterproto.string_field(6)
    # range is definition of the range of values a value is allowed
    range: List[str] = betterproto.string_field(7)
    # length is a defintion of the length restrictions for the attribute
    length: List[str] = betterproto.string_field(8)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ModelInfo(betterproto.Message):
    """ModelInfo is general information about a model plugin."""

    # name is the name given to the model plugin - no spaces and title case.
    name: str = betterproto.string_field(1)
    # version is the semantic version of the Plugin e.g. 1.0.0.
    version: str = betterproto.string_field(2)
    # model_data is a set of metadata about the YANG files that went in to
    # generating the model plugin. It includes name, version and organization for
    # each YANG file, similar to how they are represented in gNMI Capabilities.
    model_data: List["___gnmi__.ModelData"] = betterproto.message_field(3)
    # module is the name of the Model Plugin on the file system - usually ending
    # in .so.<version>.
    module: str = betterproto.string_field(4)
    # getStateMode is flag that defines how the "get state" operation works.  0)
    # means that no retrieval of state is attempted  1) means that the
    # synchronizer will make 2 requests to the device - one for      Get with
    # State and another for Get with Operational.  2) means that the synchronizer
    # will do a Get request comprising of each      one of the ReadOnlyPaths and
    # their sub paths. If there is a `list`      in any one of these paths it
    # will be sent down as is, expecting the      devices implementation of gNMI
    # will be able to expand wildcards.  3) means that the synchronizer will do a
    # Get request comprising of each      one of the ReadOnlyPaths and their sub
    # paths. If there is a `list`      in any one of these paths, a separate call
    # will be made first to find      all the instances in the list and a Get
    # including these expanded wildcards      will be sent down to the device.
    get_state_mode: int = betterproto.uint32_field(5)
    # read_only_path is all of the read only paths for the model plugin.
    read_only_path: List["ReadOnlyPath"] = betterproto.message_field(7)
    # read_write_path is all of the read write paths for the model plugin.
    read_write_path: List["ReadWritePath"] = betterproto.message_field(8)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class Chunk(betterproto.Message):
    """
    Chunk is for streaming a model plugin file to the server. There is a built
    in limit in gRPC of 4MB - plugin is usually around 20MB so break in to
    chunks of approx 1-2MB.
    """

    # so_file is the name being streamed.
    so_file: str = betterproto.string_field(1)
    # content is the bytes content.
    content: bytes = betterproto.bytes_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class RegisterResponse(betterproto.Message):
    """RegisterResponse carries status of model plugin registration."""

    # name is name of the model plugin.
    name: str = betterproto.string_field(1)
    # version is the semantic version of the model plugin.
    version: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListModelsRequest(betterproto.Message):
    """
    ListModelsRequest carries data for querying registered model plugins.
    """

    # verbose option causes all of the ReadWrite and ReadOnly paths to be
    # included.
    verbose: bool = betterproto.bool_field(1)
    # An optional filter on the name of the model plugins to list.
    model_name: str = betterproto.string_field(2)
    # An optional filter on the version of the model plugins to list
    model_version: str = betterproto.string_field(3)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class RollbackRequest(betterproto.Message):
    """
    RollbackRequest carries the name of a network config to rollback. If there
    are subsequent changes to any of the devices in that config, the rollback
    will be rejected.
    """

    # name is an optional name of a Network Change to rollback. If no name is
    # given the last network change will be rolled back. If the name given is not
    # of the last network change and error will be given.
    name: str = betterproto.string_field(1)
    # On optional comment to leave on the rollback.
    comment: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class RollbackResponse(betterproto.Message):
    """RollbackResponse carries the response of the rollback operation"""

    # A message showing the result of the rollback.
    message: str = betterproto.string_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class ListSnapshotsRequest(betterproto.Message):
    """
    ListSnapshotsRequest requests a list of snapshots for all devices and
    versions.
    """

    # subscribe indicates whether to subscribe to events (e.g. ADD, UPDATE, and
    # REMOVE) that occur after all devices have been streamed to the client
    subscribe: bool = betterproto.bool_field(1)
    # option to specify a specific device - if blank or '*' then select all Can
    # support `*` (match many chars) or '?' (match one char) as wildcard
    id: str = betterproto.string_field(2)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CompactChangesRequest(betterproto.Message):
    """
    CompactChangesRequest requests a compaction of the Network Change and
    Device Change stores
    """

    # retention_period is an optional duration of time counting back from the
    # present moment Network changes that were created during this period should
    # not be compacted Any network changes that are older should be compacted If
    # not specified the duration is 0
    retention_period: timedelta = betterproto.message_field(1)

    def __post_init__(self) -> None:
        super().__post_init__()


@dataclass(eq=False, repr=False)
class CompactChangesResponse(betterproto.Message):
    """CompactChangesResponse is a response to the Compact Changes command"""

    pass

    def __post_init__(self) -> None:
        super().__post_init__()


class ConfigAdminServiceStub(betterproto.ServiceStub):
    """
    ConfigAdminService provides means for enhanced interactions with the
    configuration subsystem.
    """

    async def upload_register_model(
        self, request_iterator: Union[AsyncIterable["Chunk"], Iterable["Chunk"]]
    ) -> "RegisterResponse":
        """
        UploadRegisterModel uploads and adds the model plugin to the list of
        supported models. The file is serialized in to Chunks of less than 4MB
        so as not to break the gRPC byte array limit
        """

        return await self._stream_unary(
            "/onos.config.admin.ConfigAdminService/UploadRegisterModel",
            request_iterator,
            Chunk,
            RegisterResponse,
        )

    async def list_registered_models(
        self, *, verbose: bool = False, model_name: str = "", model_version: str = ""
    ) -> AsyncIterator["ModelInfo"]:
        """ListRegisteredModels returns a stream of registered models."""

        request = ListModelsRequest()
        request.verbose = verbose
        request.model_name = model_name
        request.model_version = model_version

        async for response in self._unary_stream(
            "/onos.config.admin.ConfigAdminService/ListRegisteredModels",
            request,
            ModelInfo,
        ):
            yield response

    async def rollback_network_change(
        self, *, name: str = "", comment: str = ""
    ) -> "RollbackResponse":
        """
        RollbackNetworkChange rolls back the specified network change (or the
        latest one).
        """

        request = RollbackRequest()
        request.name = name
        request.comment = comment

        return await self._unary_unary(
            "/onos.config.admin.ConfigAdminService/RollbackNetworkChange",
            request,
            RollbackResponse,
        )

    async def list_snapshots(
        self, *, subscribe: bool = False, id: str = ""
    ) -> AsyncIterator["_snapshot_device__.Snapshot"]:
        """
        ListSnapshots gets a list of snapshots across all devices and versions,
        and streams them back to the caller.
        """

        request = ListSnapshotsRequest()
        request.subscribe = subscribe
        request.id = id

        async for response in self._unary_stream(
            "/onos.config.admin.ConfigAdminService/ListSnapshots",
            request,
            _snapshot_device__.Snapshot,
        ):
            yield response

    async def compact_changes(
        self, *, retention_period: timedelta = None
    ) -> "CompactChangesResponse":
        """
        CompactChanges requests a snapshot of NetworkChange and DeviceChange
        stores. This will take all of the Network Changes older than the
        retention period and flatten them down to just one snapshot (replacing
        any older snapshot). This will act as a baseline for those changes
        within the retention period and any future changes. DeviceChanges will
        be snapshotted to correspond to these NetworkChange compactions leaving
        an individual snapshot perv device and version combination.
        """

        request = CompactChangesRequest()
        if retention_period is not None:
            request.retention_period = retention_period

        return await self._unary_unary(
            "/onos.config.admin.ConfigAdminService/CompactChanges",
            request,
            CompactChangesResponse,
        )


from .... import gnmi as ___gnmi__
from ..change import device as _change_device__
from ..snapshot import device as _snapshot_device__
